@@ -1,10 +1,13 @@
+# -*- coding: utf-8 -*-
 __author__ = 'Robert Ancell <bob27@users.sourceforge.net>'
 __license__ = 'GNU General Public License Version 2'
 __copyright__ = 'Copyright 2005-2006  Robert Ancell'
 
 # Highlight types
-HIGHLIGHT_SELECTED = 'selected'
-HIGHLIGHT_CAN_MOVE = 'canMove'
+HIGHLIGHT_SELECTED   = 'selected'
+HIGHLIGHT_CAN_MOVE   = 'canMove'
+HIGHLIGHT_THREATENED = 'threatened'
+HIGHLIGHT_CAN_TAKE   = 'canTake'
 
 class ChessSet:
     """
@@ -73,7 +76,7 @@ class Scene:
         """
         pass
 
-    def addChessPiece(self, chessSet, name, coord, feedback):
+    def addChessPiece(self, chessSet, name, coord, feedback, captured = False):
         """Add a chess piece model into the scene.
         
         'chessSet' is the name of the chess set (string).
@@ -85,6 +88,14 @@ class Scene:
         """
         raise Exception('Not implemented')
 
+    def clearCapturedPieces(self):
+        """Remove all captured pieces from the scene.
+        
+        This is typically followed by calls to addChessPiece() with
+        captured = True.
+        """
+        raise Exception('Not implemented')
+
     def setBoardHighlight(self, coords):
         """Highlight a square on the board.
         
@@ -94,18 +105,28 @@ class Scene:
         """
         pass
     
-    def setBoardRotation(self, angle):
+    def setBoardRotation(self, angle, faceToFace = False, animate = True):
         """Set the rotation on the board.
         
         'angle' is the angle the board should be drawn at in degress (float, [0.0, 360.0]).
+        'faceToFace' is True if the top pieces should be drawn upside down.
+        'animate' is True if the rotation shoule be animated.
         """
         pass
     
+    def showSmooth(self, doNumbering):
+        pass
+    
     def showBoardNumbering(self, showNumbering):
         """
         """
         pass
         
+    def showCapturedPieces(self, showCaptured):
+        """
+        """
+        pass
+
     def showBoardDarker(self, showDarker):
         """
         """
