@@ -30,12 +30,6 @@
 
 #define OTHER_PLAYER(w) (((w) == WHITE_TURN) ? BLACK_TURN : WHITE_TURN)
 
-#define TILEWIDTH   80
-#define TILEHEIGHT  80
-#define GRIDWIDTH   1
-#define BOARDWIDTH  ((TILEWIDTH+GRIDWIDTH)  * 8)
-#define BOARDHEIGHT ((TILEHEIGHT+GRIDWIDTH) * 8)
-
 #define MAIN_PAGE           	0
 #define NETWORK_PAGE           	1
 
@@ -67,7 +61,7 @@ void set_selection (GtkWidget * widget, void *data);
 void free_str (GtkWidget * widget, void *data);
 void load_tiles_callback (GtkWidget * widget, void *data);
 void cancel (GtkWidget * widget, void *data);
-gint expose_event (GtkWidget * widget, GdkEventExpose * event);
+gboolean draw_event (GtkWidget * widget, cairo_t * cr);
 gint configure_event (GtkWidget * widget, GdkEventConfigure * event);
 gint button_press_event (GtkWidget * widget, GdkEventButton * event);
 void gui_draw_pixmap (gint which, gint x, gint y);
