@@ -78,7 +78,7 @@ static void games_scores_dialog_add_category (GamesScoresDialog *self,
 			 GINT_TO_POINTER (self->_priv->catcounter),
 			 k);
   self->_priv->catcounter++;
-  gtk_combo_box_append_text (GTK_COMBO_BOX (self->_priv->combo), name);
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (self->_priv->combo), name);
 }
 
 /* This is a helper function for loading the initial list of categories
@@ -104,7 +104,7 @@ static void games_scores_dialog_load_categories (GamesScoresCategory *cat,
 static void games_scores_dialog_set_style (GamesScoresDialog *self, 
 					   GamesScoreStyle style) 
 {
-  gchar *header;
+  const gchar *header;
 
   self->_priv->style = style;
   switch (style) {
@@ -210,7 +210,7 @@ static gboolean games_scores_dialog_set_edit (GamesScoresDialog *self)
   GtkTreeSelection *selection;
 
   /* Just in case we've been closed as soon as we're created. */
-  if (!GTK_WIDGET_REALIZED (self))
+  if (!gtk_widget_get_realized (GTK_WIDGET (self)))
     return FALSE;
 
   /* Temporarily disable the code that prevents editing when the
@@ -471,8 +471,6 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
   self->_priv->catcounter = 0;
   self->_priv->hilight = 0;
   self->_priv->sethilight = -1;
-
-  gtk_dialog_set_has_separator (GTK_DIALOG (self), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (self), 5);
   gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), 2);
 
@@ -512,7 +510,7 @@ static void games_scores_dialog_init (GamesScoresDialog *self)
   gtk_box_pack_start (GTK_BOX (self->_priv->catbar), self->_priv->label,
 			FALSE, FALSE, 0);	
  
-  self->_priv->combo = gtk_combo_box_new_text ();
+  self->_priv->combo = gtk_combo_box_text_new ();
   gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (self->_priv->combo), FALSE);
   gtk_box_pack_start (GTK_BOX (self->_priv->catbar), 
 			self->_priv->combo, TRUE, TRUE, 0);
