@@ -20,7 +20,6 @@
 #include <stdlib.h>
 #include <time.h>
 #include <dirent.h>
-#include <config.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -29,13 +28,13 @@
 #include <libgames-support/games-clock.h>
 #include <libgames-support/games-conf.h>
 #include <libgames-support/games-frame.h>
+#include <libgames-support/games-help.h>
 #include <libgames-support/games-stock.h>
 #include <libgames-support/games-scores.h>
 #include <libgames-support/games-scores-dialog.h>
-
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif /* HAVE_GNOME */
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-fullscreen-action.h>
+#include <libgames-support/games-pause-action.h>
     
 #include "mahjongg.h"
 #include "drawing.h"
@@ -63,65 +62,48 @@
 #define DEFAULT_TILESET "default.png"
 #define DEFAULT_MAPSET "Easy"
 
-tilepos *pos = 0;
+static GtkWidget *window, *statusbar;
+static GtkWidget *tiles_label;
+static GtkWidget *toolbar;
+static GtkWidget *moves_label, *chrono;
+static GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
+static GtkWidget *colour_well = NULL;
+static GtkWidget *pref_dialog = NULL;
 
-tile tiles[MAX_TILES];
+static GtkAction *pause_action;
+static GtkAction *hint_action;
+static GtkAction *redo_action;
+static GtkAction *undo_action;
+static GtkAction *restart_action;
+static GtkAction *scores_action;
+static GtkAction *show_toolbar_action;
+static GtkAction *fullscreen_action;
+static GtkAction *leave_fullscreen_action;
 
-GtkWidget *window, *statusbar;
-GtkWidget *tiles_label;
-GtkWidget *toolbar;
-
-GtkAction *pause_action;
-GtkAction *resume_action;
-GtkAction *hint_action;
-GtkAction *redo_action;
-GtkAction *undo_action;
-GtkAction *restart_action;
-GtkAction *scores_action;
-GtkAction *show_toolbar_action;
-GtkAction *fullscreen_action;
-GtkAction *leavefullscreen_action;
-
-gint selected_tile, visible_tiles;
-gint sequence_number;
-
-GList *tileset_list = NULL;
+/* Available tilsets */
+static GList *tileset_list = NULL;
 
 gchar *tileset = NULL;
-static gint mapset = -1;
-static gchar *score_current_mapset = NULL;
 
+tile tiles[MAX_TILES];
+static gint visible_tiles; /* Count of how many tiles are active */
+
+tilepos *pos = 0;
+static gint mapset = -1, active_mapset = -1;
 static gchar *selected_tileset = NULL;
+static gint selected_tile;
+static gint sequence_number;
+static gboolean undo_state = FALSE;
+static gboolean redo_state = FALSE;
+static gint moves_left = 0; /* The number of moves that can be made */
+gboolean paused = FALSE; /* true if the game is paused */
 
-gboolean undo_state = FALSE;
-gboolean redo_state = FALSE;
+/* Hint animation */
+static gint hint_tiles[2];
+static guint timer = 0;
+static guint timeout_counter = HINT_BLINK_NUM + 1;
 
-gint hint_tiles[2];
-guint timer;
-guint timeout_counter = HINT_BLINK_NUM + 1;
-
-GtkWidget *moves_label;
-gint moves_left = 0;
-GtkWidget *chrono;
-gint paused = 0;
-
-/* for the preferences */
-GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
-GtkWidget *colour_well = NULL;
-GtkWidget *pref_dialog = NULL;
-
-/* Has the map been changed ? */
-gboolean new_map = TRUE;
-
-GamesScores *highscores;
-
-static gint get_mapset_index (void);
-
-static const GamesScoresDescription scoredesc = { NULL,
-  "Easy",
-  "mahjongg",
-  GAMES_SCORES_STYLE_TIME_ASCENDING
-};
+static GamesScores *highscores;
 
 enum {
   GAME_RUNNING = 0,
@@ -131,16 +113,15 @@ enum {
   GAME_DEAD
 } game_over = GAME_WAITING;
 
+static gint get_mapset_index (void);
 static void clear_undo_queue (void);
 void you_won (void);
 void check_free (void);
 void undo_tile_callback (void);
 void properties_callback (void);
 void shuffle_tiles_callback (void);
-void ensure_pause_off (void);
-void pause_callback (void);
-void new_game (void);
-void restart_game (void);
+void pause_callback (GamesPauseAction *action);
+void new_game (gboolean shuffle);
 
 void
 mahjongg_theme_warning (gchar * message)
@@ -154,7 +135,7 @@ mahjongg_theme_warning (gchar * message)
 				   GTK_BUTTONS_CLOSE,
 				   _("Could not load tile set"));
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-					    message);
+					    "%s", message);
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
 				  _("Preferences"), GTK_RESPONSE_ACCEPT);
@@ -168,39 +149,7 @@ mahjongg_theme_warning (gchar * message)
   gtk_widget_destroy (dialog);
 }
 
-static void
-set_fullscreen_actions (gboolean is_fullscreen)
-{
-  gtk_action_set_sensitive (leavefullscreen_action, is_fullscreen);
-  gtk_action_set_visible (leavefullscreen_action, is_fullscreen);
-
-  gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
-  gtk_action_set_visible (fullscreen_action, !is_fullscreen);
-}
-
-static void
-fullscreen_callback (GtkAction * action)
-{
-  if (action == fullscreen_action)
-    gtk_window_fullscreen (GTK_WINDOW (window));
-  else
-    gtk_window_unfullscreen (GTK_WINDOW (window));
-}
-
-static gboolean
-window_state_callback (GtkWidget * widget, GdkEventWindowState * event)
-{
-  if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
-    return FALSE;
-
-  set_fullscreen_actions (event->new_window_state &
-			  GDK_WINDOW_STATE_FULLSCREEN);
-    
-  return FALSE;
-}
-
 /* At the end of the game, hint, shuffle and pause all become unavailable. */
-/* Undo and Redo are handled elsewhere. */
 static void
 update_menu_sensitivities (void)
 {
@@ -213,14 +162,10 @@ update_menu_sensitivities (void)
     gtk_action_set_sensitive (hint_action, FALSE);
     gtk_action_set_sensitive (undo_action, FALSE);
     gtk_action_set_sensitive (redo_action, FALSE);
-    gtk_action_set_visible (pause_action, FALSE);
-    gtk_action_set_visible (resume_action, TRUE);
   } else {
     gtk_action_set_sensitive (hint_action, moves_left > 0);
-    gtk_action_set_sensitive (undo_action, undo_state);
+    gtk_action_set_sensitive (undo_action, undo_state && game_over != GAME_WON);
     gtk_action_set_sensitive (redo_action, redo_state);
-    gtk_action_set_visible (pause_action, TRUE);
-    gtk_action_set_visible (resume_action, FALSE);
   }
 
   /* This is a workaround to make sure that all toolbar elements are
@@ -313,11 +258,9 @@ conf_value_changed_cb (GamesConf *conf,
 
     mapset = get_mapset_index ();
 
-    new_map = TRUE;
-
     /* Skip the dialog if a game isn't in play. */
     if (game_over || !games_clock_get_seconds (GAMES_CLOCK (chrono))) {
-      new_game ();
+      new_game (TRUE);
       return;
     }
 
@@ -325,17 +268,17 @@ conf_value_changed_cb (GamesConf *conf,
                                                  GTK_DIALOG_MODAL,
                                                  GTK_MESSAGE_QUESTION,
                                                  GTK_BUTTONS_NONE,
-                                                _("Do you want to finish the current game or start playing with the new map immediately?"));
+                                                _("Do you want to start a new game with this map?"));
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                               _("If you choose to finish with the old map then the next game will use the new map."));
+                                               _("If you continue playing the next game will use the new map."));
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                            _("_Finish"), GTK_RESPONSE_REJECT,
-                            GTK_STOCK_NEW, GTK_RESPONSE_ACCEPT,
+                            _("_Continue playing"), GTK_RESPONSE_REJECT,
+                            _("Use _new map"), GTK_RESPONSE_ACCEPT,
                             NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     if (response == GTK_RESPONSE_ACCEPT)
-      new_game ();
+      new_game (TRUE);
     gtk_widget_destroy (dialog);
   }
 }
@@ -438,19 +381,10 @@ message_flash (gchar * message)
     gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "flash");
   flashid =
     gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, message);
-  g_timeout_add (5000, (GSourceFunc) message_flash_remove,
+  g_timeout_add_seconds (5, (GSourceFunc) message_flash_remove,
 		 GUINT_TO_POINTER (flashid));
 }
 
-static void
-update_score_state ()
-{
-  GList *top;
-
-  top = games_scores_get (highscores);
-  gtk_action_set_sensitive (scores_action, top != NULL);
-}
-
 static gint
 update_moves_left (void)
 {
@@ -484,7 +418,7 @@ static void
 remove_pair (gint tile1, gint tile2)
 {
   gchar *tmpstr;
-
+   
   tiles[tile1].visible = tiles[tile2].visible = 0;
   tiles[tile1].selected &= ~SELECTED_FLAG;
   tiles[tile2].selected &= ~SELECTED_FLAG;
@@ -498,21 +432,20 @@ remove_pair (gint tile1, gint tile2)
   tmpstr = g_strdup_printf ("%3d", visible_tiles);
   gtk_label_set_text (GTK_LABEL (tiles_label), tmpstr);
   g_free (tmpstr);
-  update_moves_left ();
   set_undoredo_state (TRUE, FALSE);
 
+  update_moves_left ();   
   if (visible_tiles <= 0) {
     games_clock_stop (GAMES_CLOCK (chrono));
     you_won ();
   }
-
 }
 
 void
 tile_event (gint tileno, gint button)
 {
   if (paused) {
-    pause_callback ();
+    pause_callback (NULL);
     return;
   }
 
@@ -554,7 +487,8 @@ fill_tile_menu (GtkWidget * menu)
   struct dirent *e;
   DIR *dir;
   gint itemno = 0;
-  gchar *dname = NULL;
+  const char *dname = NULL;
+  gchar **tileset_name;
 
   if (tileset_list) {
     g_list_foreach (tileset_list, (GFunc) g_free, NULL);
@@ -563,11 +497,10 @@ fill_tile_menu (GtkWidget * menu)
 
   tileset_list = NULL;
 
-  dname = g_build_filename (PIXMAPDIR, NULL);
+  dname = games_runtime_get_directory(GAMES_RUNTIME_GAME_PIXMAP_DIRECTORY);
   dir = opendir (dname);
 
   if (!dir) {
-    g_free (dname);
     return;
   }
 
@@ -583,10 +516,8 @@ fill_tile_menu (GtkWidget * menu)
       continue;
     }
       
-    // TODO: Strip off suffix (bug #406856)
-
-    gchar** tileset_name = g_strsplit (s, ".", -1);
-    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), tileset_name[0]);
+    tileset_name = g_strsplit (s, ".", -1);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (menu), tileset_name[0]);
     g_strfreev (tileset_name);
 
     tileset_list = g_list_append (tileset_list, s);
@@ -598,7 +529,6 @@ fill_tile_menu (GtkWidget * menu)
   }
 
   closedir (dir);
-  g_free (dname);
 }
 
 static void
@@ -607,7 +537,10 @@ fill_map_menu (GtkWidget * menu)
   gint lp;
 
   for (lp = 0; lp < nmaps; lp++) {
-    gtk_combo_box_append_text (GTK_COMBO_BOX (menu), Q_ (maps[lp].name));
+    const char *display_name;
+
+    display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[lp].name);
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (menu), display_name);
   }
   gtk_combo_box_set_active (GTK_COMBO_BOX (menu), mapset);
 }
@@ -636,6 +569,8 @@ check_free (void)
 
     update_menu_sensitivities ();
     if (!game_over) {
+      gint response_id;
+       
       mb = gtk_message_dialog_new (GTK_WINDOW (window),
 				   GTK_DIALOG_MODAL
 				   | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -643,14 +578,27 @@ check_free (void)
 				   GTK_BUTTONS_NONE,
 				   (_("There are no more moves.")));
       gtk_dialog_add_buttons (GTK_DIALOG (mb),
-			      GTK_STOCK_UNDO,
-			      GTK_RESPONSE_REJECT,
-			      _("Shuffle"), GTK_RESPONSE_ACCEPT, NULL);
+			      GTK_STOCK_UNDO, GTK_RESPONSE_REJECT,
+                              _("_New game"), GTK_RESPONSE_CANCEL, NULL);
+
+      /* Can only shuffle if two tiles are visible to be matched. This cannot occur
+       * if all tiles are in a stack */
+      if (visible_tiles >= 2)
+        gtk_dialog_add_button(GTK_DIALOG (mb), _("_Shuffle"), GTK_RESPONSE_ACCEPT);
+
       gtk_dialog_set_default_response (GTK_DIALOG (mb), GTK_RESPONSE_ACCEPT);
-      if (gtk_dialog_run (GTK_DIALOG (mb)) == GTK_RESPONSE_ACCEPT)
-	shuffle_tiles_callback ();
-      else
-	undo_tile_callback ();
+      response_id = gtk_dialog_run (GTK_DIALOG (mb));
+      switch (response_id) {
+      case GTK_RESPONSE_ACCEPT:
+	 shuffle_tiles_callback ();
+         break;
+      case GTK_RESPONSE_CANCEL:
+         new_game (TRUE);
+         break;
+      default:
+	 undo_tile_callback ();
+         break;
+      }       
 
       gtk_widget_destroy (mb);
     }
@@ -683,6 +631,7 @@ you_won (void)
 						    (dialog), _("Map:"));
       message =
 	g_strdup_printf ("<b>%s</b>\n\n%s", _("Congratulations!"),
+                         pos == 1 ? _("Your score is the best!") :
 			 _("Your score has made the top ten."));
       games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
       g_free (message);
@@ -693,7 +642,6 @@ you_won (void)
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_hide (dialog);
   }
-  update_score_state ();
 }
 
 static void
@@ -725,8 +673,7 @@ properties_callback (void)
 					     GTK_STOCK_CLOSE,
 					     GTK_RESPONSE_CLOSE, NULL);
   gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), 2);
-  gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
+  gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (pref_dialog))), 2);
   gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
   gtk_dialog_set_default_response (GTK_DIALOG (pref_dialog),
 				   GTK_RESPONSE_CLOSE);
@@ -752,7 +699,7 @@ properties_callback (void)
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
 		    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
 
-  omenu = gtk_combo_box_new_text ();
+  omenu = gtk_combo_box_text_new ();
   fill_tile_menu (omenu);
   g_signal_connect (G_OBJECT (omenu), "changed",
 		    G_CALLBACK (tileset_callback), NULL);
@@ -773,7 +720,7 @@ properties_callback (void)
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
 		    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
 
-  omenu = gtk_combo_box_new_text ();
+  omenu = gtk_combo_box_text_new ();
   fill_map_menu (omenu);
   g_signal_connect (G_OBJECT (omenu), "changed",
 		    G_CALLBACK (set_map_selection), NULL);
@@ -803,8 +750,8 @@ properties_callback (void)
 
   gtk_container_add (GTK_CONTAINER (frame), table);
 
-  gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox),
-			       top_table);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (pref_dialog))),
+			       top_table, TRUE, TRUE, 0);
 
   g_object_unref (group);
   gtk_widget_show_all (pref_dialog);
@@ -846,7 +793,8 @@ stop_hints (void)
 static void
 hint_callback (void)
 {
-  gint i, j, free = 0, type;
+  gint i, j, type;
+  gboolean have_match = FALSE;
 
   if (paused || (game_over != GAME_RUNNING && game_over != GAME_WAITING))
     return;
@@ -860,29 +808,47 @@ hint_callback (void)
    * Tile Free is now _so_ much quicker, it is more elegant to do a
    * British Library search, and safer. */
   /* Note: British Library should probably read British Museum.  */
-
-  /* Clear any selection */
+   
+  /* Check if the selected tile has a match */
   if (selected_tile < MAX_TILES) {
-    tiles[selected_tile].selected = 0;
+    type = tiles[selected_tile].type;
+    for (i = 0; i < MAX_TILES && !have_match; i++) {
+       if (tiles[i].type == type && i != selected_tile && tile_free (i)) {
+         have_match = TRUE;
+         hint_tiles[0] = selected_tile;
+         hint_tiles[1] = i;
+       }
+    }
+  }
+
+  /* Check if any tiles match */
+  for (i = 0; i < MAX_TILES && !have_match; i++) {
+    if (tile_free (i)) {
+      type = tiles[i].type;
+      for (j = 0; j < MAX_TILES && !have_match; j++) {
+	if (tiles[j].type == type && i != j && tile_free (j)) {
+          have_match = TRUE;
+          hint_tiles[0] = i;
+          hint_tiles[1] = j;
+        }
+      }
+    }
+  }
+   
+  if (!have_match)
+    return;
+
+  /* Clear selection if no part of the hint */
+  if (selected_tile < MAX_TILES && hint_tiles[0] != selected_tile) {
+    tiles[selected_tile].selected &= ~SELECTED_FLAG;
     draw_tile (selected_tile);
     selected_tile = MAX_TILES + 1;
   }
 
-  for (i = 0; i < MAX_TILES && !free; i++)
-    if (tile_free (i)) {
-      type = tiles[i].type;
-      for (j = 0; j < MAX_TILES && !free; j++) {
-	free = (tiles[j].type == type && i != j && tile_free (j));
-	if (free) {
-	  tiles[i].selected ^= HINT_FLAG;
-	  tiles[j].selected ^= HINT_FLAG;
-	  draw_tile (i);
-	  draw_tile (j);
-	  hint_tiles[0] = i;
-	  hint_tiles[1] = j;
-	}
-      }
-    }
+  tiles[hint_tiles[0]].selected |= HINT_FLAG;
+  tiles[hint_tiles[1]].selected |= HINT_FLAG;
+  draw_tile (hint_tiles[0]);
+  draw_tile (hint_tiles[1]);
 
   /* This is a good way to test check_free
    * for (i=0;i<MAX_TILES;i++)
@@ -893,8 +859,8 @@ hint_callback (void)
   timer = g_timeout_add (250, (GSourceFunc) hint_timeout, NULL);
 
   /* 30s penalty */
-  games_clock_add_seconds (GAMES_CLOCK (chrono), 30);
   clock_start ();
+  games_clock_add_seconds (GAMES_CLOCK (chrono), 30);
 }
 
 static void
@@ -955,7 +921,7 @@ about_callback (void)
 }
 
 void
-pause_callback (void)
+pause_callback (GamesPauseAction *action)
 {
   static gboolean noloops = FALSE;
 
@@ -967,7 +933,8 @@ pause_callback (void)
 
   noloops = TRUE;
   stop_hints ();
-  paused = !paused;
+  if (action)
+    paused = games_pause_action_get_is_paused (action);
   draw_all_tiles ();
   update_menu_sensitivities ();
   if (paused) {
@@ -978,15 +945,6 @@ pause_callback (void)
     message ("");
   }
   noloops = FALSE;
-
-
-}
-
-void
-ensure_pause_off (void)
-{
-  if (paused)
-    pause_callback ();
 }
 
 static void
@@ -1006,43 +964,16 @@ scores_callback (GtkAction * action, gpointer data)
   gtk_widget_hide (dialog);
 }
 
-static void
-init_game (void)
-{
-  gchar *newtitle;
-
-  newtitle = g_strdup_printf (_("%s - %s"), _("Mahjongg"),
-			      Q_ (maps[mapset].name));
-  gtk_window_set_title (GTK_WINDOW (window), newtitle);
-  g_free (newtitle);
-
-  score_current_mapset = maps[mapset].score_name;
-
-  gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
-  update_moves_left ();
-  game_over = GAME_WAITING;
-  sequence_number = 1;
-  visible_tiles = MAX_TILES;
-  selected_tile = MAX_TILES + 1;
-  set_undoredo_state (FALSE, FALSE);
-
-  games_clock_stop (GAMES_CLOCK (chrono));
-  games_clock_set_seconds (GAMES_CLOCK (chrono), 0);
-}
-
 static void
 new_game_cb (GtkAction * action, gpointer data)
 {
-  stop_hints ();
-  ensure_pause_off ();
-  new_game ();
+  new_game (TRUE);
 }
 
 static void
 restart_game_cb (GtkAction * action, gpointer data)
 {
-  stop_hints ();
-  restart_game ();
+  new_game (FALSE);
 }
 
 static void
@@ -1051,21 +982,6 @@ quit_cb (GObject * object, gpointer data)
   gtk_main_quit ();
 }
 
-void
-restart_game (void)
-{
-  gint i;
-
-  ensure_pause_off ();
-  for (i = 0; i < MAX_TILES; i++) {
-    tiles[i].visible = 1;
-    tiles[i].selected = 0;
-    tiles[i].sequence = 0;
-  }
-  draw_all_tiles ();
-  init_game ();
-}
-
 static void
 redo_tile_callback (GtkAction * action, gpointer data)
 {
@@ -1109,6 +1025,7 @@ redo_tile_callback (GtkAction * action, gpointer data)
       break;
     }
   }
+
   set_undoredo_state (TRUE, found);
 
   update_moves_left ();
@@ -1164,25 +1081,6 @@ clear_undo_queue (void)
       tiles[lp].sequence = 0;
 }
 
-static void
-load_map (void)
-{
-  new_map = FALSE;
-  pos = maps[mapset].map;
-  generate_dependencies ();
-  calculate_view_geometry ();
-  configure_pixmaps ();
-}
-
-static void
-do_game (void)
-{
-  games_scores_set_category (highscores, maps[mapset].score_name);
-  if (new_map)
-    load_map ();
-  generate_game (g_random_int ());	/* puts in the positions of the tiles */
-}
-
 static void
 load_preferences (void)
 {
@@ -1200,22 +1098,74 @@ load_preferences (void)
 }
 
 void
-new_game (void)
+new_game (gboolean shuffle)
 {
-  do_game ();
+  gint i;
+  gchar *title;
+  const char *display_name;
 
+  /* Reset state */
+  game_over = GAME_WAITING;
+  sequence_number = 1;
+  visible_tiles = MAX_TILES;
+  selected_tile = MAX_TILES + 1;
+  undo_state = FALSE;
+  redo_state = FALSE;
+  hint_tiles[0] = MAX_TILES + 1;
+  hint_tiles[1] = MAX_TILES + 1;   
+  if (timer)
+    g_source_remove (timer);
+  timer = 0;
+  timeout_counter = HINT_BLINK_NUM + 1;
+  moves_left = 0;
+  paused = FALSE;
+  for (i = 0; i < MAX_TILES; i++) {
+    tiles[i].visible = 1;
+    tiles[i].selected = 0;
+    tiles[i].sequence = 0;
+  }
+
+  /* Load map */
+  if (active_mapset != mapset) {
+    active_mapset = mapset;
+    pos = maps[mapset].map;
+    generate_dependencies ();
+    calculate_view_geometry ();
+    configure_pixmaps ();
+    games_scores_set_category (highscores, maps[mapset].score_name);
+  }
+
+  /* Generate layout */
+  if (shuffle) {
+     generate_game (g_random_int ());
+  }
+   
+  /* Analyse layout */
+  update_moves_left ();
+  
+  /* Update menus */
+  update_menu_sensitivities ();
+
+  /* Set window title */
+  /* Translators: This is the window title for Mahjongg which contains the map name, e.g. 'Mahjongg - Red Dragon' */
+
+  display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[mapset].name);
+  title = g_strdup_printf (_("Mahjongg - %s"), display_name);
+  gtk_window_set_title (GTK_WINDOW (window), title);
+  g_free (title);
+  gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
+
+  /* Prepare clock */
+  games_clock_stop (GAMES_CLOCK (chrono));
+  games_clock_reset (GAMES_CLOCK (chrono));
+
+  /* Redraw */
   draw_all_tiles ();
-
-  init_game ();
-
-  update_score_state ();
 }
 
 void
 shuffle_tiles_callback (void)
 {
-  gboolean ok;
-
   if (paused || game_over == GAME_DEAD || game_over == GAME_WON)
     return;
 
@@ -1226,36 +1176,23 @@ shuffle_tiles_callback (void)
     unselect_tile (selected_tile);
   }
 
-  ok = shuffle ();
+  /* Shuffle the tiles - this should always succeed, the option should not be
+   * available if they cannot be shuffled */
+  if (!shuffle ())
+    return;
 
-  if (!ok) {
-    GtkWidget *mb;
-    game_over = GAME_DEAD;
-    games_clock_stop (GAMES_CLOCK (chrono));
-    mb = gtk_message_dialog_new (GTK_WINDOW (window),
-				 GTK_DIALOG_MODAL |
-				 GTK_DIALOG_DESTROY_WITH_PARENT,
-				 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-				 (_
-				  ("Sorry, I was unable to find a playable configuration.")));
-    gtk_dialog_run (GTK_DIALOG (mb));
-    gtk_widget_destroy (mb);
+  draw_all_tiles ();
 
-  } else {
+  game_over = GAME_RUNNING;
 
-    draw_all_tiles ();
+  /* 60s penalty */
+  games_clock_add_seconds (GAMES_CLOCK (chrono), 60);
 
-    game_over = GAME_RUNNING;
-
-    /* 60s penalty */
-    games_clock_add_seconds (GAMES_CLOCK (chrono), 60);
-
-    update_moves_left ();
-    /* Disable undo/redo after a shuffle. */
-    sequence_number = 1;
-    clear_undo_queue ();
-    set_undoredo_state (FALSE, FALSE);
-  }
+  update_moves_left ();
+  /* Disable undo/redo after a shuffle. */
+  sequence_number = 1;
+  clear_undo_queue ();
+  set_undoredo_state (FALSE, FALSE);
 
   update_menu_sensitivities ();
 }
@@ -1263,11 +1200,7 @@ shuffle_tiles_callback (void)
 static void
 help_cb (GtkAction * action, gpointer data)
 {
-#ifdef HAVE_GNOME
-  gnome_help_display ("mahjongg.xml", NULL, NULL);
-#else
-#warning need to port help to gtk-only!
-#endif /* HAVE_GNOME */
+  games_help_display (window, APPNAME, NULL);
 }
 
 static const GtkActionEntry actions[] = {
@@ -1278,10 +1211,6 @@ static const GtkActionEntry actions[] = {
    G_CALLBACK (new_game_cb)},
   {"RestartGame", GAMES_STOCK_RESTART_GAME, NULL, NULL,
    N_("Restart the current game"), G_CALLBACK (restart_game_cb)},
-  {"PauseGame", GAMES_STOCK_PAUSE_GAME, NULL, NULL, N_("Pause the game"),
-   G_CALLBACK (pause_callback)},
-  {"ResumeGame", GAMES_STOCK_RESUME_GAME, NULL, NULL,
-   N_("Resume the paused game"), G_CALLBACK (pause_callback)},
   {"UndoMove", GAMES_STOCK_UNDO_MOVE, NULL, NULL, N_("Undo the last move"),
    G_CALLBACK (undo_tile_callback)},
   {"RedoMove", GAMES_STOCK_REDO_MOVE, NULL, NULL, N_("Redo the last move"),
@@ -1291,10 +1220,6 @@ static const GtkActionEntry actions[] = {
   {"Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL,
    G_CALLBACK (scores_callback)},
   {"Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_cb)},
-  {"Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_callback)},
-  {"LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_callback)},
   {"Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL,
    G_CALLBACK (properties_callback)},
   {"Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb)},
@@ -1313,7 +1238,6 @@ static const char ui_description[] =
   "      <menuitem action='NewGame'/>"
   "      <menuitem action='RestartGame'/>"
   "      <menuitem action='PauseGame'/>"
-  "      <menuitem action='ResumeGame'/>"
   "      <separator/>"
   "      <menuitem action='UndoMove'/>"
   "      <menuitem action='RedoMove'/>"
@@ -1325,7 +1249,6 @@ static const char ui_description[] =
   "    </menu>"
   "    <menu action='SettingsMenu'>"
   "      <menuitem action='Fullscreen'/>"
-  "      <menuitem action='LeaveFullscreen'/>"
   "      <menuitem action='ShowToolbar'/>"
   "      <separator/>"
   "      <menuitem action='Preferences'/>"
@@ -1337,13 +1260,9 @@ static const char ui_description[] =
   "  </menubar>"
   "  <toolbar name='Toolbar'>"
   "    <toolitem action='NewGame'/>"
-  "    <toolitem action='RestartGame'/>"
-  "    <toolitem action='PauseGame'/>"
-  "    <toolitem action='ResumeGame'/>"
-  "    <separator/>"
   "    <toolitem action='UndoMove'/>"
-  "    <toolitem action='RedoMove'/>"
   "    <toolitem action='Hint'/>"
+  "    <toolitem action='PauseGame'/>"
   "    <toolitem action='LeaveFullscreen'/>"
   "  </toolbar>"
   "</ui>";
@@ -1365,23 +1284,27 @@ create_menus (GtkUIManager * ui_manager)
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
   gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
   restart_action = gtk_action_group_get_action (action_group, "RestartGame");
-  pause_action = gtk_action_group_get_action (action_group, "PauseGame");
-  resume_action = gtk_action_group_get_action (action_group, "ResumeGame");
+  pause_action = GTK_ACTION (games_pause_action_new ("PauseGame"));
+  gtk_action_set_is_important (pause_action, TRUE);
+  g_signal_connect (G_OBJECT (pause_action), "state-changed", G_CALLBACK (pause_callback), NULL);
+  gtk_action_group_add_action_with_accel (action_group, pause_action, NULL);
   hint_action = gtk_action_group_get_action (action_group, "Hint");
+  gtk_action_set_is_important (hint_action, TRUE);
   undo_action = gtk_action_group_get_action (action_group, "UndoMove");
+  gtk_action_set_is_important (undo_action, TRUE);
   redo_action = gtk_action_group_get_action (action_group, "RedoMove");
   scores_action = gtk_action_group_get_action (action_group, "Scores");
   show_toolbar_action =
     gtk_action_group_get_action (action_group, "ShowToolbar");
 
-  fullscreen_action =
-    gtk_action_group_get_action (action_group, "Fullscreen");
-  leavefullscreen_action =
-    gtk_action_group_get_action (action_group, "LeaveFullscreen");
-  set_fullscreen_actions (FALSE);
+  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW (window)));
+  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
+
+  leave_fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("LeaveFullscreen", GTK_WINDOW (window)));
+  gtk_action_group_add_action_with_accel (action_group, leave_fullscreen_action, NULL);
 
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (show_toolbar_action),
-                                games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL));
+                                games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL));  
 }
 
 static void
@@ -1389,10 +1312,17 @@ init_scores (void)
 {
   int i;
 
-  highscores = games_scores_new (&scoredesc);
+  highscores = games_scores_new ("mahjongg",
+                                 NULL, 0,
+                                 NULL, NULL,
+                                 0 /* default category; FIXMEchpe: was: "Easy" */,
+                                 GAMES_SCORES_STYLE_TIME_ASCENDING);
 
   for (i = 0; i < nmaps; i++) {
-    games_scores_add_category (highscores, maps[i].score_name, maps[i].name);
+    const char *display_name;
+
+    display_name = g_dpgettext2 (NULL, "mahjongg map name", maps[i].name);
+    games_scores_add_category (highscores, maps[i].score_name, display_name);
   }
 }
 
@@ -1412,40 +1342,21 @@ main (int argc, char *argv[])
 
   GOptionContext *context;
 
-#ifdef HAVE_GNOME
-  GnomeClient *client;
-  GnomeProgram *program;
-#else
   gboolean retval;
   GError *error = NULL;
-#endif
 
-#if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
-  /* If we're going to use gnome-vfs, we need to init threads before
-   * calling any glib functions.
-   */
-  g_thread_init (NULL);
-#endif
+  if (!games_runtime_init ("mahjongg"))
+    return 1;
 
+#ifdef ENABLE_SETGID
   setgid_io_init ();
-
-  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-  textdomain (GETTEXT_PACKAGE);
+#endif
 
   context = g_option_context_new (NULL);
 #if GLIB_CHECK_VERSION (2, 12, 0)
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 #endif
 
-#ifdef HAVE_GNOME
-  program = gnome_program_init (APPNAME, VERSION,
-				LIBGNOMEUI_MODULE,
-				argc, argv,
-                                GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PARAM_APP_DATADIR, DATADIR,
-                                NULL);
-#else
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
@@ -1455,7 +1366,6 @@ main (int argc, char *argv[])
     g_error_free (error);
     exit (1);
   }
-#endif /* HAVE_GNOME */
 
   g_set_application_name (_(APPNAME_LONG));
 
@@ -1482,9 +1392,6 @@ main (int argc, char *argv[])
 
   load_preferences ();
 
-  g_signal_connect (window, "window-state-event",
-		    G_CALLBACK (window_state_callback), NULL);
-
   /* Statusbar for a chrono, Tiles left and Moves left */
   status_box = gtk_hbox_new (FALSE, 10);
 
@@ -1520,10 +1427,9 @@ main (int argc, char *argv[])
   ui_manager = gtk_ui_manager_new ();
 
   games_stock_prepare_for_statusbar_tooltips (ui_manager, statusbar);
-  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (statusbar), FALSE);
 
   create_menus (ui_manager);
-  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+  accel_group = gtk_ui_manager_get_accel_group (ui_manager);  
   gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
   box = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 
@@ -1540,17 +1446,15 @@ main (int argc, char *argv[])
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), board, TRUE, TRUE, 0);
 
-  gtk_box_pack_end (GTK_BOX (statusbar), status_box, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (statusbar), status_box, FALSE, FALSE, 6);
   gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
   /* FIXME: get these in the best place (as per the comment below. */
   init_config ();
-
-  do_game ();
-  init_game ();
-  update_score_state ();
+   
+  new_game (TRUE);
 
   /* Don't leave the keyboard focus on the toolbar */
   gtk_widget_grab_focus (board);
@@ -1559,6 +1463,7 @@ main (int argc, char *argv[])
    * window knows how big to make the tiles. */
   gtk_widget_show_all (window);
 
+  games_fullscreen_action_set_visible_policy (GAMES_FULLSCREEN_ACTION (leave_fullscreen_action), GAMES_FULLSCREEN_ACTION_VISIBLE_ON_FULLSCREEN);
   if (!games_conf_get_boolean (NULL, KEY_SHOW_TOOLBAR, NULL))
     gtk_widget_hide (toolbar);
 
@@ -1568,9 +1473,7 @@ main (int argc, char *argv[])
 
   games_conf_shutdown ();
 
-#ifdef HAVE_GNOME
-  g_object_unref (program);
-#endif /* HAVE_GNOME */
+  games_runtime_shutdown ();
 
   return 0;
 }
